use ExtUtils::MakeMaker;
use strict;
use warnings;

main();
exit 0;

sub main {
    my $fixed_args = fixed_args('Ogre');
    my $varied_args = varied_args();

    WriteMakefile(%$fixed_args, %$varied_args);
}

sub fixed_args {
    my ($pkg) = @_;

    return {
        'NAME'          => $pkg,
        'VERSION_FROM'  => "$pkg.pm",
        'ABSTRACT_FROM' => "$pkg.pm",
        'AUTHOR'        => 'Scott Lanning <slanning@cpan.org>',
        'CC'            => $ENV{'CXX'} || 'g++',
        'LD'            => '$(CC)',
        'OBJECT'        => '$(O_FILES)',
        'MAN3PODS'      => {},
        'XSOPT'         => '-C++',
        'TYPEMAPS'      => ['perlobject.map'],
    };
}

sub varied_args {
    # Make sure libs are known by pkg-config
    my @errors = ();
    push @errors, check_pkg_config('OGRE', '1.4.3');
    if (@errors) {
        die(map { "$_$/" } @errors);
    }

    # Get include dirs and defines
    my @cflags = ();
    push @cflags, pkg_config('OGRE', 'cflags');

    # Get lib dirs
    my @libs = ();
    push @libs, pkg_config('OGRE', 'libs');

    return {
        'PREREQ_PM'         => {
            'Test::More' => 0,

            # these are required for some tutorials
            # 'OIS' => 0.03,
            # 'Readonly' => 0,
        },
        'INC'               => join(' ', @cflags),
        'LIBS'              => join(' ', @libs),
    };
}

sub check_pkg_config {
    my ($pkg, $required_version) = @_;

    my $pc = 'pkg-config';
    my @errors = ();

    # Check that pkg-config is installed
    my $pcver = `$pc --version`;
    if ($pcver eq '') {
        push @errors, "$pc not found";
    }
    else {
        # Check that pkg-config knows about the package
        my $pkgexists = `$pc --exists $pkg`;
        if ($?) {
            push @errors, "Package $pkg not found by $pc";
        }
        else {
            # Check that the package is the right version
            my $pkgver = `$pc --atleast-version='$required_version' $pkg`;
            if ($?) {
                push @errors, "Package $pkg is not the right version (at least $required_version)";
            }
        }
    }

    return @errors;
}

sub pkg_config {
    my ($pkg, $option) = @_;

    my $pc = 'pkg-config';

    my $str = `$pc --$option $pkg`;
    if ($?) {
        die "$pc --$option $pkg: $str\n";
    }
    else {
        chomp $str;
        return $str;
    }
}
